import { Request, Response } from "express";
import logger from "../logger/logger";
import User from "../models/user";
import { isPasswordValid, isUsernameValid } from "../validator/validator";
import { Roles } from "../utils/constants";
import {
  decodeBase64,
  hashPassword,
  newUserId,
  saveSession,
} from "../utils/utils";

export async function SignUp(req: Request, res: Response) {
  try {
    const { username, password } = req.body;

    // validate request

    if (!username || !password) {
      return res.status(400).send({ err: "invalid request" });
    }

    if (!isUsernameValid(username)) {
      return res.status(400).send({ err: "invalid request" });
    }

    // check if user already exists

    const existingUser = await User.findOne({
      where: {
        username: username,
      },
    });

    if (existingUser) {
      logger.debug("User already exists with this username: %s", username);
      return res.status(400).send({ err: "invalid request" });
    }

    // decode password

    const decodedPassword = decodeBase64(password);

    if (!isPasswordValid(decodedPassword)) {
      logger.debug("Password is not valid");
      return res.status(400).send({ err: "invalid request" });
    }

    const hashedPassword = await hashPassword(decodedPassword);

    // create user

    await User.create({
      user_id: newUserId(),
      role: Roles.Master,
      username: username,
      password: hashedPassword,
    })
      .then((user) => {
        logger.debug("User created with username: %s", user.username);

        saveSession(res, user.user_id, user.username);
      })
      .catch((err) => {
        logger.error(err);
        res.status(500).send({ err: "invalid request" });
      });
  } catch (error) {
    logger.error(error);
    res.status(500).send({ err: "invalid request" });
  }
}
